@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 from dataclasses import dataclass
+from enum import Enum
 from typing import Any
 
 from schemathesis.config._checks import ChecksConfig
@@ -17,7 +18,7 @@ class PhaseConfig(DiffBase):
     generation: GenerationConfig
     checks: ChecksConfig
 
-    __slots__ = ("enabled", "generation", "checks")
+    __slots__ = ("enabled", "generation", "checks", "_is_default")
 
     def __init__(
         self,
@@ -29,6 +30,7 @@ class PhaseConfig(DiffBase):
         self.enabled = enabled
         self.generation = generation or GenerationConfig()
         self.checks = checks or ChecksConfig()
+        self._is_default = enabled and generation is None and checks is None
 
     @classmethod
     def from_dict(cls, data: dict[str, Any]) -> PhaseConfig:
@@ -46,7 +48,7 @@ class ExamplesPhaseConfig(DiffBase):
     generation: GenerationConfig
     checks: ChecksConfig
 
-    __slots__ = ("enabled", "fill_missing", "generation", "checks")
+    __slots__ = ("enabled", "fill_missing", "generation", "checks", "_is_default")
 
     def __init__(
         self,
@@ -60,6 +62,7 @@ class ExamplesPhaseConfig(DiffBase):
         self.fill_missing = fill_missing
         self.generation = generation or GenerationConfig()
         self.checks = checks or ChecksConfig()
+        self._is_default = enabled and not fill_missing and generation is None and checks is None
 
     @classmethod
     def from_dict(cls, data: dict[str, Any]) -> ExamplesPhaseConfig:
@@ -79,7 +82,14 @@ class CoveragePhaseConfig(DiffBase):
     checks: ChecksConfig
     unexpected_methods: set[str]
 
-    __slots__ = ("enabled", "generate_duplicate_query_parameters", "generation", "checks", "unexpected_methods")
+    __slots__ = (
+        "enabled",
+        "generate_duplicate_query_parameters",
+        "generation",
+        "checks",
+        "unexpected_methods",
+        "_is_default",
+    )
 
     def __init__(
         self,
@@ -95,6 +105,13 @@ class CoveragePhaseConfig(DiffBase):
         self.unexpected_methods = unexpected_methods or DEFAULT_UNEXPECTED_METHODS
         self.generation = generation or GenerationConfig()
         self.checks = checks or ChecksConfig()
+        self._is_default = (
+            enabled
+            and not generate_duplicate_query_parameters
+            and generation is None
+            and checks is None
+            and unexpected_methods is None
+        )
 
     @classmethod
     def from_dict(cls, data: dict[str, Any]) -> CoveragePhaseConfig:
@@ -109,9 +126,14 @@ class CoveragePhaseConfig(DiffBase):
         )
 
 
+class InferenceAlgorithm(str, Enum):
+    LOCATION_HEADERS = "location-headers"
+    DEPENDENCY_ANALYSIS = "dependency-analysis"
+
+
 @dataclass(repr=False)
 class InferenceConfig(DiffBase):
-    algorithms: list[str]
+    algorithms: list[InferenceAlgorithm]
 
     __slots__ = ("algorithms",)
 
@@ -120,12 +142,14 @@ class InferenceConfig(DiffBase):
         *,
         algorithms: list[str] | None = None,
     ) -> None:
-        self.algorithms = algorithms if algorithms is not None else ["location-headers"]
+        self.algorithms = (
+            [InferenceAlgorithm(a) for a in algorithms] if algorithms is not None else list(InferenceAlgorithm)
+        )
 
     @classmethod
     def from_dict(cls, data: dict[str, Any]) -> InferenceConfig:
         return cls(
-            algorithms=data.get("algorithms", ["location-headers"]),
+            algorithms=data.get("algorithms", list(InferenceAlgorithm)),
         )
 
     @property
@@ -133,6 +157,9 @@ class InferenceConfig(DiffBase):
         """Inference is enabled if any algorithms are configured."""
         return bool(self.algorithms)
 
+    def is_algorithm_enabled(self, algorithm: InferenceAlgorithm) -> bool:
+        return algorithm in self.algorithms
+
 
 @dataclass(repr=False)
 class StatefulPhaseConfig(DiffBase):
@@ -142,7 +169,7 @@ class StatefulPhaseConfig(DiffBase):
     max_steps: int
     inference: InferenceConfig
 
-    __slots__ = ("enabled", "generation", "checks", "max_steps", "inference")
+    __slots__ = ("enabled", "generation", "checks", "max_steps", "inference", "_is_default")
 
     def __init__(
         self,
@@ -158,6 +185,7 @@ class StatefulPhaseConfig(DiffBase):
         self.generation = generation or GenerationConfig()
         self.checks = checks or ChecksConfig()
         self.inference = inference or InferenceConfig()
+        self._is_default = enabled and generation is None and checks is None and max_steps is None and inference is None
 
     @classmethod
     def from_dict(cls, data: dict[str, Any]) -> StatefulPhaseConfig:
