@@ -347,6 +347,8 @@ class ProjectConfig(DiffBase):
             for op in self.operations.operations:
                 if op._filter_set.applies_to(operation=operation):
                     configs.append(op.phases)
+        if not configs:
+            return self.phases
         configs.append(self.phases)
         return PhasesConfig.from_hierarchy(configs)
 
@@ -367,7 +369,10 @@ class ProjectConfig(DiffBase):
         if phase is not None:
             phases = self.phases_for(operation=operation)
             phase_config = phases.get_by_name(name=phase)
-            configs.append(phase_config.generation)
+            if not phase_config._is_default:
+                configs.append(phase_config.generation)
+        if not configs:
+            return self.generation
         configs.append(self.generation)
         return GenerationConfig.from_hierarchy(configs)
 
@@ -388,7 +393,10 @@ class ProjectConfig(DiffBase):
         if phase is not None:
             phases = self.phases_for(operation=operation)
             phase_config = phases.get_by_name(name=phase)
-            configs.append(phase_config.checks)
+            if not phase_config._is_default:
+                configs.append(phase_config.checks)
+        if not configs:
+            return self.checks
         configs.append(self.checks)
         return ChecksConfig.from_hierarchy(configs)
 
