@@ -17,6 +17,7 @@ if TYPE_CHECKING:
 
     from schemathesis.config import OutputConfig
     from schemathesis.core.compat import RefResolutionError
+    from schemathesis.core.jsonschema import BundleError
 
 
 SCHEMA_ERROR_SUGGESTION = "Ensure that the definition complies with the OpenAPI specification"
@@ -26,11 +27,6 @@ SERIALIZATION_NOT_POSSIBLE_MESSAGE = f"No supported serializers for media types:
 SERIALIZATION_FOR_TYPE_IS_NOT_POSSIBLE_MESSAGE = (
     f"Cannot serialize to '{{}}' (unsupported media type)\n{SERIALIZERS_SUGGESTION_MESSAGE}"
 )
-RECURSIVE_REFERENCE_ERROR_MESSAGE = (
-    "Currently, Schemathesis can't generate data for this operation due to "
-    "recursive references in the operation definition. See more information in "
-    "this issue - https://github.com/schemathesis/schemathesis/issues/947"
-)
 
 
 class SchemathesisError(Exception):
@@ -50,6 +46,14 @@ class InvalidSchema(SchemathesisError):
         self.path = path
         self.method = method
 
+    @classmethod
+    def from_bundle_error(cls, error: BundleError, location: str, name: str | None = None) -> InvalidSchema:
+        if location == "body":
+            message = f"Can not generate data for {location}! {error}"
+        else:
+            message = f"Can not generate data for {location} parameter `{name}`! {error}"
+        return InvalidSchema(message)
+
     @classmethod
     def from_jsonschema_error(
         cls, error: ValidationError | JsonSchemaError, path: str | None, method: str | None, config: OutputConfig
@@ -283,6 +287,30 @@ class UnboundPrefix(SerializationError):
         super().__init__(UNBOUND_PREFIX_MESSAGE_TEMPLATE.format(prefix=prefix))
 
 
+class UnresolvableReference(SchemathesisError):
+    """A reference cannot be resolved."""
+
+    def __init__(self, reference: str) -> None:
+        self.reference = reference
+
+    def __str__(self) -> str:
+        return f"Reference `{self.reference}` cannot be resolved"
+
+
+class InfiniteRecursiveReference(SchemathesisError):
+    """A schema has required references forming an infinite cycle."""
+
+    def __init__(self, reference: str, cycle: list[str]) -> None:
+        self.reference = reference
+        self.cycle = cycle
+
+    def __str__(self) -> str:
+        if len(self.cycle) == 1:
+            return f"Schema `{self.reference}` has a required reference to itself"
+        cycle_str = " ->\n  ".join(self.cycle + [self.cycle[0]])
+        return f"Schema `{self.reference}` has required references forming a cycle:\n\n  {cycle_str}"
+
+
 class SerializationNotPossible(SerializationError):
     """Not possible to serialize data to specified media type(s).
 
