@@ -4,12 +4,13 @@ import time
 from dataclasses import dataclass
 from typing import TYPE_CHECKING, Any
 
-from schemathesis.config import ProjectConfig
+from schemathesis.config import InferenceAlgorithm, ProjectConfig
 from schemathesis.core import NOT_SET, NotSet
 from schemathesis.engine.control import ExecutionControl
 from schemathesis.engine.observations import Observations
 from schemathesis.generation.case import Case
 from schemathesis.schemas import APIOperation, BaseSchema
+from schemathesis.specs.openapi.stateful import dependencies
 
 if TYPE_CHECKING:
     import threading
@@ -85,9 +86,10 @@ class EngineContext:
 
     def inject_links(self) -> int:
         """Inject inferred OpenAPI links into API operations based on collected observations."""
+        from schemathesis.specs.openapi.schemas import BaseOpenAPISchema
+
         injected = 0
         if self.observations is not None and self.observations.location_headers:
-            from schemathesis.specs.openapi.schemas import BaseOpenAPISchema
             from schemathesis.specs.openapi.stateful.inference import LinkInferencer
 
             assert isinstance(self.schema, BaseOpenAPISchema)
@@ -95,8 +97,15 @@ class EngineContext:
             # Generate links from collected Location headers
             inferencer = LinkInferencer.from_schema(self.schema)
             for operation, entries in self.observations.location_headers.items():
-                injected += inferencer.inject_links(operation.definition.raw, entries)
-
+                injected += inferencer.inject_links(operation.responses, entries)
+        if (
+            isinstance(self.schema, BaseOpenAPISchema)
+            and self.schema.config.phases.stateful.enabled
+            and self.schema.config.phases.stateful.inference.is_algorithm_enabled(
+                InferenceAlgorithm.DEPENDENCY_ANALYSIS
+            )
+        ):
+            injected += dependencies.inject_links(self.schema)
         return injected
 
     def stop(self) -> None:
