@@ -8,7 +8,6 @@ from warnings import WarningMessage, catch_warnings
 
 import requests
 from hypothesis.errors import InvalidArgument
-from hypothesis_jsonschema._canonicalise import HypothesisRefResolutionError
 from jsonschema.exceptions import SchemaError as JsonSchemaError
 from jsonschema.exceptions import ValidationError
 from requests.exceptions import ChunkedEncodingError
@@ -37,7 +36,6 @@ from schemathesis.engine.errors import (
     TestingState,
     UnexpectedError,
     UnrecoverableNetworkError,
-    UnsupportedRecursiveReference,
     clear_hypothesis_notes,
     deduplicate_errors,
     is_unrecoverable_network_error,
@@ -47,10 +45,12 @@ from schemathesis.engine.recorder import ScenarioRecorder
 from schemathesis.generation import metrics, overrides
 from schemathesis.generation.case import Case
 from schemathesis.generation.hypothesis.builder import (
+    InfiniteRecursiveReferenceMark,
     InvalidHeadersExampleMark,
     InvalidRegexMark,
     MissingPathParameters,
     NonSerializableMark,
+    UnresolvableReferenceMark,
     UnsatisfiableExampleMark,
 )
 from schemathesis.generation.hypothesis.reporting import ignore_hypothesis_output
@@ -173,14 +173,24 @@ def run_test(
         status = Status.ERROR
         try:
             operation.schema.validate()
-            msg = "Unexpected error during testing of this API operation"
-            exc_msg = str(exc)
-            if exc_msg:
-                msg += f": {exc_msg}"
-            try:
-                raise InternalError(msg) from exc
-            except InternalError as exc:
-                yield non_fatal_error(exc)
+            # JSON Schema validation can miss it if there is `$ref` adjacent to `type` on older specifications
+            if str(exc).startswith("Unknown type"):
+                yield non_fatal_error(
+                    InvalidSchema(
+                        message=str(exc),
+                        path=operation.path,
+                        method=operation.method,
+                    )
+                )
+            else:
+                msg = "Unexpected error during testing of this API operation"
+                exc_msg = str(exc)
+                if exc_msg:
+                    msg += f": {exc_msg}"
+                try:
+                    raise InternalError(msg) from exc
+                except InternalError as exc:
+                    yield non_fatal_error(exc)
         except ValidationError as exc:
             yield non_fatal_error(
                 InvalidSchema.from_jsonschema_error(
@@ -190,9 +200,6 @@ def run_test(
                     config=ctx.config.output,
                 )
             )
-    except HypothesisRefResolutionError:
-        status = Status.ERROR
-        yield non_fatal_error(UnsupportedRecursiveReference())
     except InvalidArgument as exc:
         status = Status.ERROR
         message = get_invalid_regular_expression_message(warnings)
@@ -265,6 +272,16 @@ def run_test(
         status = Status.ERROR
         yield non_fatal_error(missing_path_parameters)
 
+    infinite_recursive_reference = InfiniteRecursiveReferenceMark.get(test_function)
+    if infinite_recursive_reference:
+        status = Status.ERROR
+        yield non_fatal_error(infinite_recursive_reference)
+
+    unresolvable_reference = UnresolvableReferenceMark.get(test_function)
+    if unresolvable_reference:
+        status = Status.ERROR
+        yield non_fatal_error(unresolvable_reference)
+
     for error in deduplicate_errors(errors):
         yield non_fatal_error(error)
 
@@ -368,7 +385,7 @@ def test_func(
     transport_kwargs: dict[str, Any],
     continue_on_failure: bool,
 ) -> None:
-    recorder.record_case(parent_id=None, transition=None, case=case)
+    recorder.record_case(parent_id=None, case=case, transition=None, is_transition_applied=False)
     try:
         response = case.call(**transport_kwargs)
     except (requests.Timeout, requests.ConnectionError, ChunkedEncodingError) as error:
