@@ -3,6 +3,7 @@
 from schemathesis.core.errors import (
     HookError,
     IncorrectUsage,
+    InfiniteRecursiveReference,
     InternalError,
     InvalidHeadersExample,
     InvalidRateLimit,
@@ -19,11 +20,13 @@ from schemathesis.core.errors import (
     SerializationNotPossible,
     TransitionValidationError,
     UnboundPrefix,
+    UnresolvableReference,
 )
 
 __all__ = [
     "HookError",
     "IncorrectUsage",
+    "InfiniteRecursiveReference",
     "InternalError",
     "InvalidHeadersExample",
     "InvalidRateLimit",
@@ -40,4 +43,5 @@ __all__ = [
     "SerializationNotPossible",
     "TransitionValidationError",
     "UnboundPrefix",
+    "UnresolvableReference",
 ]
