@@ -37,12 +37,16 @@ class StepInput:
 
     case: Case
     transition: Transition | None  # None for initial steps
+    # Whether this transition was actually applied
+    # Data extraction failures can prevent it, as well as transitions can be skipped in some cases
+    # to improve discovery of bugs triggered by non-stateful inputs during stateful testing
+    is_applied: bool
 
-    __slots__ = ("case", "transition")
+    __slots__ = ("case", "transition", "is_applied")
 
     @classmethod
     def initial(cls, case: Case) -> StepInput:
-        return cls(case=case, transition=None)
+        return cls(case=case, transition=None, is_applied=False)
 
 
 @dataclass
@@ -52,10 +56,11 @@ class Transition:
     # ID of the transition (e.g. link name)
     id: str
     parent_id: str
+    is_inferred: bool
     parameters: dict[str, dict[str, ExtractedParam]]
     request_body: ExtractedParam | None
 
-    __slots__ = ("id", "parent_id", "parameters", "request_body")
+    __slots__ = ("id", "parent_id", "is_inferred", "parameters", "request_body")
 
 
 @dataclass
