@@ -14,7 +14,6 @@ from jsonschema.exceptions import SchemaError
 
 from schemathesis.core.control import SkipTest
 from schemathesis.core.errors import (
-    RECURSIVE_REFERENCE_ERROR_MESSAGE,
     SERIALIZERS_SUGGESTION_MESSAGE,
     IncorrectUsage,
     InvalidHeadersExample,
@@ -147,7 +146,7 @@ class SchemathesisCase(PyCollector):
                 if override is not None:
                     for location, entry in override.items():
                         if entry:
-                            as_strategy_kwargs[location] = entry
+                            as_strategy_kwargs[location.container_name] = entry
                 modes = []
                 phases = self.schema.config.phases_for(operation=operation)
                 if phases.examples.enabled:
@@ -293,8 +292,6 @@ def pytest_pyfunc_call(pyfuncitem):  # type:ignore
 
     For example - kwargs validation is failed for some strategy.
     """
-    from hypothesis_jsonschema._canonicalise import HypothesisRefResolutionError
-
     from schemathesis.generation.hypothesis.builder import (
         InvalidHeadersExampleMark,
         InvalidRegexMark,
@@ -312,8 +309,6 @@ def pytest_pyfunc_call(pyfuncitem):  # type:ignore
             if "Inconsistent args" in str(exc) and "@example()" in str(exc):
                 raise IncorrectUsage(GIVEN_AND_EXPLICIT_EXAMPLES_ERROR_MESSAGE) from None
             raise InvalidSchema(exc.args[0]) from None
-        except HypothesisRefResolutionError:
-            pytest.skip(RECURSIVE_REFERENCE_ERROR_MESSAGE)
         except (SkipTest, unittest.SkipTest) as exc:
             if UnsatisfiableExampleMark.is_set(pyfuncitem.obj):
                 raise Unsatisfiable("Failed to generate test cases from examples for this API operation") from None
