@@ -18,7 +18,8 @@ from urllib.parse import quote, unquote, urljoin, urlsplit, urlunsplit
 
 from schemathesis import transport
 from schemathesis.config import ProjectConfig
-from schemathesis.core import NOT_SET, NotSet
+from schemathesis.core import NOT_SET, NotSet, media_types
+from schemathesis.core.adapter import OperationParameter, ResponsesContainer
 from schemathesis.core.errors import IncorrectUsage, InvalidSchema
 from schemathesis.core.result import Ok, Result
 from schemathesis.core.transport import Response
@@ -302,9 +303,6 @@ class BaseSchema(Mapping):
     def get_strategies_from_examples(self, operation: APIOperation, **kwargs: Any) -> list[SearchStrategy[Case]]:
         raise NotImplementedError
 
-    def get_security_requirements(self, operation: APIOperation) -> list[str]:
-        raise NotImplementedError
-
     def get_parameter_serializer(self, operation: APIOperation, location: str) -> Callable | None:
         raise NotImplementedError
 
@@ -432,21 +430,12 @@ class BaseSchema(Mapping):
         """
         raise NotImplementedError
 
-    def get_links(self, operation: APIOperation) -> dict[str, dict[str, Any]]:
-        raise NotImplementedError
-
     def get_tags(self, operation: APIOperation) -> list[str] | None:
         raise NotImplementedError
 
     def validate_response(self, operation: APIOperation, response: Response) -> bool | None:
         raise NotImplementedError
 
-    def prepare_schema(self, schema: Any) -> Any:
-        raise NotImplementedError
-
-    def _get_payload_schema(self, definition: dict[str, Any], media_type: str) -> dict[str, Any] | None:
-        raise NotImplementedError
-
     def as_strategy(
         self,
         generation_mode: GenerationMode = GenerationMode.POSITIVE,
@@ -514,39 +503,7 @@ class APIOperationMap(Mapping):
         return strategies.combine(_strategies)
 
 
-@dataclass(eq=False)
-class Parameter:
-    """A logically separate parameter bound to a location (e.g., to "query string").
-
-    For example, if the API requires multiple headers to be present, each header is presented as a separate
-    `Parameter` instance.
-    """
-
-    # The parameter definition in the language acceptable by the API
-    definition: Any
-
-    __slots__ = ("definition",)
-
-    @property
-    def location(self) -> str:
-        """Where this parameter is located.
-
-        E.g. "query" or "body"
-        """
-        raise NotImplementedError
-
-    @property
-    def name(self) -> str:
-        """Parameter name."""
-        raise NotImplementedError
-
-    @property
-    def is_required(self) -> bool:
-        """Whether the parameter is required for a successful API call."""
-        raise NotImplementedError
-
-
-P = TypeVar("P", bound=Parameter)
+P = TypeVar("P", bound=OperationParameter)
 
 
 @dataclass
@@ -572,14 +529,11 @@ class ParameterSet(Generic[P]):
                 return parameter
         return None
 
-    def contains(self, name: str) -> bool:
-        return self.get(name) is not None
-
-    def __contains__(self, item: str) -> bool:
-        return self.contains(item)
-
-    def __bool__(self) -> bool:
-        return bool(self.items)
+    def __contains__(self, name: str) -> bool:
+        for parameter in self.items:
+            if parameter.name == name:
+                return True
+        return False
 
     def __iter__(self) -> Generator[P, None, None]:
         yield from iter(self.items)
@@ -595,6 +549,8 @@ class PayloadAlternatives(ParameterSet[P]):
     """A set of alternative payloads."""
 
 
+R = TypeVar("R", bound=ResponsesContainer)
+S = TypeVar("S")
 D = TypeVar("D", bound=dict)
 
 
@@ -608,16 +564,14 @@ class OperationDefinition(Generic[D]):
     """
 
     raw: D
-    resolved: D
-    scope: str
 
-    __slots__ = ("raw", "resolved", "scope")
+    __slots__ = ("raw",)
 
     def _repr_pretty_(self, *args: Any, **kwargs: Any) -> None: ...
 
 
 @dataclass()
-class APIOperation(Generic[P]):
+class APIOperation(Generic[P, R, S]):
     """An API operation (e.g., `GET /users`)."""
 
     # `path` does not contain `basePath`
@@ -627,6 +581,8 @@ class APIOperation(Generic[P]):
     method: str
     definition: OperationDefinition = field(repr=False)
     schema: BaseSchema
+    responses: R
+    security: S
     label: str = None  # type: ignore
     app: Any = None
     base_url: str | None = None
@@ -640,7 +596,7 @@ class APIOperation(Generic[P]):
         if self.label is None:
             self.label = f"{self.method.upper()} {self.path}"  # type: ignore
 
-    def __deepcopy__(self, memo: dict) -> APIOperation[P]:
+    def __deepcopy__(self, memo: dict) -> APIOperation[P, R, S]:
         return self
 
     def __hash__(self) -> int:
@@ -655,10 +611,6 @@ class APIOperation(Generic[P]):
     def full_path(self) -> str:
         return self.schema.get_full_path(self.path)
 
-    @property
-    def links(self) -> dict[str, dict[str, Any]]:
-        return self.schema.get_links(self)
-
     @property
     def tags(self) -> list[str] | None:
         return self.schema.get_tags(self)
@@ -690,6 +642,13 @@ class APIOperation(Generic[P]):
             return container.get(name)
         return None
 
+    def get_bodies_for_media_type(self, media_type: str) -> Iterator[P]:
+        main_target, sub_target = media_types.parse(media_type)
+        for body in self.body:
+            main, sub = media_types.parse(body.media_type)  # type:ignore[attr-defined]
+            if main in ("*", main_target) and sub in ("*", sub_target):
+                yield body
+
     def as_strategy(
         self,
         generation_mode: GenerationMode = GenerationMode.POSITIVE,
@@ -739,9 +698,6 @@ class APIOperation(Generic[P]):
             strategy = _apply_hooks(hooks, strategy)
         return strategy
 
-    def get_security_requirements(self) -> list[str]:
-        return self.schema.get_security_requirements(self)
-
     def get_strategies_from_examples(self, **kwargs: Any) -> list[SearchStrategy[Case]]:
         return self.schema.get_strategies_from_examples(self, **kwargs)
 
@@ -839,9 +795,3 @@ class APIOperation(Generic[P]):
             return True
         except AssertionError:
             return False
-
-    def get_raw_payload_schema(self, media_type: str) -> dict[str, Any] | None:
-        return self.schema._get_payload_schema(self.definition.raw, media_type)
-
-    def get_resolved_payload_schema(self, media_type: str) -> dict[str, Any] | None:
-        return self.schema._get_payload_schema(self.definition.resolved, media_type)
