@@ -8,19 +8,20 @@ import jsonschema
 from hypothesis import strategies as st
 from hypothesis.stateful import Bundle, Rule, precondition, rule
 
-from schemathesis.core.errors import InvalidStateMachine
+from schemathesis.core.errors import InvalidStateMachine, InvalidTransition
+from schemathesis.core.parameters import ParameterLocation
 from schemathesis.core.result import Ok
 from schemathesis.core.transforms import UNRESOLVABLE
 from schemathesis.engine.recorder import ScenarioRecorder
 from schemathesis.generation import GenerationMode
 from schemathesis.generation.case import Case
 from schemathesis.generation.hypothesis import strategies
-from schemathesis.generation.meta import ComponentInfo, ComponentKind, TestPhase
+from schemathesis.generation.meta import ComponentInfo, TestPhase
 from schemathesis.generation.stateful import STATEFUL_TESTS_LABEL
 from schemathesis.generation.stateful.state_machine import APIStateMachine, StepInput, StepOutput, _normalize_name
 from schemathesis.schemas import APIOperation
 from schemathesis.specs.openapi.stateful.control import TransitionController
-from schemathesis.specs.openapi.stateful.links import OpenApiLink, get_all_links
+from schemathesis.specs.openapi.stateful.links import OpenApiLink
 from schemathesis.specs.openapi.utils import expand_status_code
 
 if TYPE_CHECKING:
@@ -48,6 +49,8 @@ class OpenAPIStateMachine(APIStateMachine):
 
 # The proportion of negative tests generated for "root" transitions
 NEGATIVE_TEST_CASES_THRESHOLD = 10
+# How often some transition is skipped
+USE_TRANSITION_THRESHOLD = 85
 
 
 @dataclass
@@ -97,12 +100,14 @@ def collect_transitions(operations: list[APIOperation]) -> ApiTransitions:
     selected_labels = {operation.label for operation in operations}
     errors = []
     for operation in operations:
-        for _, link in get_all_links(operation):
-            if isinstance(link, Ok):
-                if link.ok().target.label in selected_labels:
-                    transitions.add_outgoing(operation.label, link.ok())
-            else:
-                errors.append(link.err())
+        for status_code, response in operation.responses.items():
+            for name, link in response.iter_links():
+                try:
+                    link = OpenApiLink(name, status_code, link, operation)
+                    if link.target.label in selected_labels:
+                        transitions.add_outgoing(operation.label, link)
+                except InvalidTransition as exc:
+                    errors.append(exc)
 
     if errors:
         raise InvalidStateMachine(errors)
@@ -118,7 +123,7 @@ def create_state_machine(schema: BaseOpenAPISchema) -> type[APIStateMachine]:
 
     # Create bundles and matchers
     for operation in operations:
-        all_status_codes = tuple(operation.definition.raw["responses"])
+        all_status_codes = operation.responses.status_codes
         bundle_matchers = []
 
         if operation.label in transitions.operations:
@@ -223,7 +228,7 @@ def classify_root_transitions(operations: list[APIOperation], transitions: ApiTr
         if not operation_transitions or not operation_transitions.outgoing:
             continue
 
-        if is_likely_root_transition(operation, operation_transitions):
+        if is_likely_root_transition(operation):
             roots.reliable.add(operation.label)
         else:
             roots.fallback.add(operation.label)
@@ -231,7 +236,7 @@ def classify_root_transitions(operations: list[APIOperation], transitions: ApiTr
     return roots
 
 
-def is_likely_root_transition(operation: APIOperation, transitions: OperationTransitions) -> bool:
+def is_likely_root_transition(operation: APIOperation) -> bool:
     """Check if operation is likely to succeed as a root transition."""
     # POST operations with request bodies are likely to create resources
     if operation.method == "post" and operation.body:
@@ -266,8 +271,12 @@ def into_step_input(
                 and isinstance(transition.request_body.value, Ok)
                 and transition.request_body.value.ok() is not UNRESOLVABLE
                 and not link.merge_body
+                and draw(st.integers(min_value=0, max_value=99)) < USE_TRANSITION_THRESHOLD
             ):
                 kwargs["body"] = transition.request_body.value.ok()
+
+            is_applied = bool(kwargs)
+
             cases = strategies.combine(
                 [target.as_strategy(generation_mode=mode, phase=TestPhase.STATEFUL, **kwargs) for mode in modes]
             )
@@ -277,22 +286,26 @@ def into_step_input(
                 and isinstance(transition.request_body.value, Ok)
                 and transition.request_body.value.ok() is not UNRESOLVABLE
                 and link.merge_body
+                and draw(st.integers(min_value=0, max_value=99)) < USE_TRANSITION_THRESHOLD
             ):
                 new = transition.request_body.value.ok()
                 if isinstance(case.body, dict) and isinstance(new, dict):
                     case.body = {**case.body, **new}
                 else:
                     case.body = new
+                is_applied = True
                 if case.meta and case.meta.generation.mode == GenerationMode.NEGATIVE:
                     # It is possible that the new body is now valid and the whole test case could be valid too
                     for alternative in case.operation.body:
                         if alternative.media_type == case.media_type:
-                            schema = alternative.as_json_schema(case.operation)
-                            if jsonschema.validators.validator_for(schema)(schema).is_valid(new):
-                                case.meta.components[ComponentKind.BODY] = ComponentInfo(mode=GenerationMode.POSITIVE)
+                            schema = alternative.optimized_schema
+                            if jsonschema.validators.validator_for(schema)(schema).is_valid(case.body):
+                                case.meta.components[ParameterLocation.BODY] = ComponentInfo(
+                                    mode=GenerationMode.POSITIVE
+                                )
                                 if all(info.mode == GenerationMode.POSITIVE for info in case.meta.components.values()):
                                     case.meta.generation.mode = GenerationMode.POSITIVE
-            return StepInput(case=case, transition=transition)
+            return StepInput(case=case, transition=transition, is_applied=is_applied)
 
         return inner(output=_output)
 
@@ -317,10 +330,13 @@ def transition(*, name: str, target: Bundle, input: st.SearchStrategy[StepInput]
     def step_function(self: OpenAPIStateMachine, input: StepInput) -> StepOutput | None:
         if input.transition is not None:
             self.recorder.record_case(
-                parent_id=input.transition.parent_id, transition=input.transition, case=input.case
+                parent_id=input.transition.parent_id,
+                transition=input.transition,
+                case=input.case,
+                is_transition_applied=input.is_applied,
             )
         else:
-            self.recorder.record_case(parent_id=None, transition=None, case=input.case)
+            self.recorder.record_case(parent_id=None, case=input.case, transition=None, is_transition_applied=False)
         self.control.record_step(input, self.recorder)
         return APIStateMachine._step(self, input=input)
 
