@@ -9,7 +9,7 @@ from urllib.parse import urlparse
 
 from schemathesis.core import NotSet
 from schemathesis.core.rate_limit import ratelimit
-from schemathesis.core.transforms import deepclone, merge_at
+from schemathesis.core.transforms import merge_at
 from schemathesis.core.transport import DEFAULT_RESPONSE_TIMEOUT, Response
 from schemathesis.generation.overrides import Override
 from schemathesis.transport import BaseTransport, SerializationContext
@@ -61,7 +61,7 @@ class RequestsTransport(BaseTransport["requests.Session"]):
 
         # Replace empty dictionaries with empty strings, so the parameters actually present in the query string
         if any(value == {} for value in (params or {}).values()):
-            params = deepclone(params)
+            params = dict(params)
             for key, value in params.items():
                 if value == {}:
                     params[key] = ""
@@ -243,7 +243,6 @@ def multipart_serializer(ctx: SerializationContext, value: Any) -> dict[str, Any
     if isinstance(value, bytes):
         return {"data": value}
     if isinstance(value, dict):
-        value = deepclone(value)
         multipart = _prepare_form_data(value)
         files, data = ctx.case.operation.prepare_multipart(multipart)
         return {"files": files, "data": data}
@@ -256,14 +255,7 @@ def multipart_serializer(ctx: SerializationContext, value: Any) -> dict[str, Any
 
 @REQUESTS_TRANSPORT.serializer("application/xml", "text/xml")
 def xml_serializer(ctx: SerializationContext, value: Any) -> dict[str, Any]:
-    media_type = ctx.case.media_type
-
-    assert media_type is not None
-
-    raw_schema = ctx.case.operation.get_raw_payload_schema(media_type)
-    resolved_schema = ctx.case.operation.get_resolved_payload_schema(media_type)
-
-    return serialize_xml(value, raw_schema, resolved_schema)
+    return serialize_xml(ctx.case, value)
 
 
 @REQUESTS_TRANSPORT.serializer("application/x-www-form-urlencoded")
