@@ -272,12 +272,37 @@ def test_unsatisfiable(cli, schema_url, workers, snapshot_cli):
 
 
 @pytest.mark.operations("invalid")
-@pytest.mark.parametrize("workers", [1])
-def test_invalid_operation(cli, schema_url, workers, snapshot_cli):
+def test_invalid_operation(cli, schema_url, snapshot_cli):
     # When the app's schema contains errors
     # For example if its type is "int" but should be "integer"
     # And schema validation is disabled
-    assert cli.run(schema_url, f"--workers={workers}", "--phases=fuzzing", "--mode=positive") == snapshot_cli
+    assert cli.run(schema_url, "--phases=fuzzing", "--mode=positive") == snapshot_cli
+
+
+def test_invalid_type_with_ref(cli, ctx, openapi3_base_url, snapshot_cli):
+    schema_path = ctx.openapi.write_schema(
+        {
+            "/test": {
+                "post": {
+                    "parameters": [
+                        {
+                            "in": "header",
+                            "name": "h",
+                            "schema": {
+                                "$ref": "#/components/schemas/S",
+                                "type": "invalid",
+                            },
+                        },
+                    ],
+                    "responses": {"default": {"description": "Ok"}},
+                }
+            }
+        },
+        components={"schemas": {"S": {"maxProperties": 5}}},
+    )
+    assert (
+        cli.run(str(schema_path), f"--url={openapi3_base_url}", "--phases=fuzzing", "--mode=positive") == snapshot_cli
+    )
 
 
 @pytest.mark.operations("teapot")
@@ -475,7 +500,15 @@ def test_register_check(new_check, cli, schema_url, snapshot_cli):
 @pytest.mark.filterwarnings("ignore:Exception in thread")
 def test_keyboard_interrupt(cli, schema_url, base_url, mocker, swagger_20, workers, snapshot_cli):
     # When a Schemathesis run in interrupted by keyboard or via SIGINT
-    operation = APIOperation("/success", "GET", {}, swagger_20, base_url=base_url)
+    operation = APIOperation(
+        "/success",
+        "GET",
+        {},
+        swagger_20,
+        base_url=base_url,
+        responses=swagger_20._parse_responses({}, ""),
+        security=swagger_20._parse_security({}),
+    )
     original = operation.Case().call
     counter = 0
 
@@ -1325,13 +1358,10 @@ def test_invalid_schema_with_disabled_validation(
     assert cli.run(str(schema_path), f"--url={openapi3_base_url}") == snapshot_cli
 
 
-def test_unresolvable_reference_with_disabled_validation(
-    ctx, cli, open_api_3_schema_with_recoverable_errors, snapshot_cli, openapi3_base_url
-):
+def test_unresolvable_reference(ctx, cli, open_api_3_schema_with_recoverable_errors, snapshot_cli, openapi3_base_url):
     # When there is an error in the schema
     del open_api_3_schema_with_recoverable_errors["paths"]["/bar"]["get"]
     schema_path = ctx.makefile(open_api_3_schema_with_recoverable_errors)
-    # And the validation is disabled (default)
     # Then we should show an error message derived from JSON Schema
     assert cli.run(str(schema_path), f"--url={openapi3_base_url}") == snapshot_cli
 
@@ -1766,6 +1796,63 @@ def test_recursive_reference_error_message(ctx, cli, schema_with_recursive_refer
     assert cli.run(str(schema_path), f"--url={openapi3_base_url}", "--mode=positive") == snapshot_cli
 
 
+def test_nullable_reference(ctx, cli, openapi3_base_url, snapshot_cli):
+    schema_path = ctx.openapi.write_schema(
+        {
+            "/test": {
+                "get": {
+                    "parameters": [
+                        {
+                            "in": "query",
+                            "name": "q",
+                            "schema": {
+                                "$ref": "#/components/schemas/MessageStatus",
+                                "nullable": True,
+                            },
+                        }
+                    ],
+                    "responses": {"default": {"description": "Ok"}},
+                }
+            }
+        },
+        components={"schemas": {"MessageStatus": {}}},
+    )
+    assert (
+        cli.run(str(schema_path), f"--url={openapi3_base_url}", "--mode=positive", "--phases=fuzzing") == snapshot_cli
+    )
+
+
+def test_reference_in_examples(ctx, cli, openapi3_base_url, snapshot_cli):
+    schema_path = ctx.openapi.write_schema(
+        {
+            "/test": {
+                "post": {
+                    "parameters": [
+                        {"$ref": "#components/parameters/metadata"},
+                        {"$ref": "#components/parameters/applicationId"},
+                    ]
+                }
+            }
+        },
+        components={
+            "parameters": {
+                "applicationId": {
+                    "example": 0,
+                    "in": "header",
+                    "name": "xd",
+                    "schema": {},
+                },
+                "metadata": {
+                    "content": {"text/plain": {"schema": {"$ref": "t"}}},
+                    "in": "header",
+                    "name": "xa",
+                },
+            }
+        },
+    )
+    assert cli.run(str(schema_path), f"--url={openapi3_base_url}", "--phases=examples") == snapshot_cli
+
+
 @pytest.mark.openapi_version("3.0")
 @pytest.mark.operations("payload")
 @pytest.mark.skipif(sys.version_info >= (3, 13), reason="Traceback is different")
