@@ -1,3 +1,4 @@
+import platform
 import textwrap
 
 import pytest
@@ -85,3 +86,11 @@ def test_incorrect_config(cli, snapshot_cli, tmp_path, config_content):
 
 def test_non_existing_file(cli, snapshot_cli):
     assert cli.main("--config-file=unknown-file.toml", "run", "http://127.0.0.1") == snapshot_cli
+
+
+@pytest.mark.skipif(platform.system() == "Windows", reason="Has different error message on Windows")
+def test_with_null_byte(cli, snapshot_cli):
+    assert (
+        cli.main("run", "http://127.0.0.1", config={"reports": {"junit": {"enabled": True, "path": "\x00"}}})
+        == snapshot_cli
+    )
