@@ -302,6 +302,7 @@ class CliSnapshotConfig:
     replace_test_cases: bool = True
     replace_phase_statistic: bool = False
     remove_last_line: bool = False
+    replace: bool = True
 
     @classmethod
     def from_request(cls, request: FixtureRequest) -> CliSnapshotConfig:
@@ -315,6 +316,8 @@ class CliSnapshotConfig:
         return self.request.getfixturevalue("testdir")
 
     def serialize(self, data: str) -> str:
+        if not self.replace:
+            return data
         if self.replace_test_cases:
             data = re.sub(r"Test cases:\n  (\d+) generated, \1 skipped", "Test cases:\n  N generated, N skipped", data)
             # Cases with failures and skips
@@ -348,7 +351,7 @@ class CliSnapshotConfig:
                         pass
             with keep_cwd():
                 data = data.replace(Path(self.testdir.tmpdir).as_uri(), "file:///tmp")
-        data = re.sub(r"http://127\.0\.0\.1:[0-9]{3,}/", "http://127.0.0.1/", data)
+        data = re.sub(r"http://127\.0\.0\.1:[0-9]{3,}", "http://127.0.0.1", data)
         if self.replace_tmp_dir:
             with keep_cwd():
                 data = data.replace(str(self.testdir.tmpdir) + os.path.sep, "/tmp/")
@@ -970,6 +973,13 @@ def openapi_30():
     return schemathesis.openapi.from_dict(raw)
 
 
+@pytest.fixture
+def openapi_31():
+    raw = make_schema("simple_openapi.yaml")
+    raw["openapi"] = "3.1.0"
+    return schemathesis.openapi.from_dict(raw)
+
+
 @pytest.fixture
 def app_schema(openapi_version, operations):
     return openapi._aiohttp.make_openapi_schema(operations=operations, version=openapi_version)
