@@ -14,10 +14,17 @@ from requests.models import RequestEncodingMixin
 import schemathesis
 from schemathesis.config._projects import ProjectConfig
 from schemathesis.core import NOT_SET
+from schemathesis.core.errors import MalformedMediaType
+from schemathesis.core.parameters import LOCATION_TO_CONTAINER
+from schemathesis.core.result import Ok
 from schemathesis.generation import GenerationMode
-from schemathesis.generation.hypothesis.builder import HypothesisTestConfig, HypothesisTestMode, create_test
+from schemathesis.generation.hypothesis.builder import (
+    HypothesisTestConfig,
+    HypothesisTestMode,
+    _iter_coverage_cases,
+    create_test,
+)
 from schemathesis.generation.meta import TestPhase
-from schemathesis.specs.openapi.constants import LOCATION_TO_CONTAINER
 from test.utils import assert_requests_call
 
 
@@ -999,6 +1006,73 @@ def test_underspecified_path_parameters(ctx, cli, snapshot_cli, openapi3_base_ur
     )
 
 
+def test_path_parameters_arent_missing(ctx, cli, snapshot_cli, openapi3_base_url):
+    # When `--mode=negative`, still generate path parameters if they can't be negated
+    schema_path = ctx.openapi.write_schema(
+        {
+            "/organizations/{organization_id}/": {
+                "get": {
+                    "parameters": [
+                        {
+                            "name": "organization_id",
+                            "in": "path",
+                            "required": True,
+                            "schema": {"type": "string"},
+                        },
+                        {
+                            "name": "q",
+                            "in": "query",
+                            "required": True,
+                            "schema": {"type": "integer", "minimum": 10},
+                        },
+                    ],
+                    "responses": {"200": {"description": "Successful Response"}},
+                }
+            }
+        }
+    )
+    assert (
+        cli.run(
+            str(schema_path),
+            f"--url={openapi3_base_url}",
+            "--checks=not_a_server_error",
+            "--phases=coverage",
+            "--mode=negative",
+        )
+        == snapshot_cli
+    )
+
+
+@pytest.mark.filterwarnings("error")
+def test_path_parameters_without_schema(ctx, cli, snapshot_cli, openapi2_base_url):
+    schema_path = ctx.openapi.write_schema(
+        {
+            "/{param}": {
+                "put": {
+                    "parameters": [
+                        {
+                            "in": "path",
+                            "name": "param",
+                            "x-custom": 0,
+                        }
+                    ],
+                }
+            }
+        },
+        version="2.0",
+    )
+    assert (
+        cli.run(
+            str(schema_path),
+            f"--url={openapi2_base_url}",
+            "--checks=not_a_server_error",
+            "--phases=coverage",
+            "--mode=negative",
+        )
+        == snapshot_cli
+    )
+
+
 def test_path_parameter_dots(ctx):
     schema = build_schema(
         ctx,
@@ -1041,6 +1115,10 @@ def test_path_parameter_dots(ctx):
                 {"path_parameters": {"name": "null%2Cnull"}},
                 {"path_parameters": {"name": ANY}},
             ],
+            [
+                {"path_parameters": {"name": "null%2Cnull"}},
+                {"path_parameters": {"name": "null"}},
+            ],
         ),
     )
 
@@ -1244,6 +1322,33 @@ def test_path_parameter(ctx):
     )
 
 
+def test_path_parameter_as_string_non_empty(ctx):
+    schema = build_schema(
+        ctx,
+        [
+            {"name": "id", "in": "path", "required": True, "schema": {"type": "string", "minLength": 1}},
+        ],
+        path="/foo/{id}",
+    )
+    assert_coverage(
+        schema,
+        list(GenerationMode),
+        [
+            {
+                "path_parameters": {
+                    "id": "00",
+                },
+            },
+            {
+                "path_parameters": {
+                    "id": "0",
+                },
+            },
+        ],
+        path=("/foo/{id}", "post"),
+    )
+
+
 def test_incorrect_headers_with_loose_schema(ctx):
     schema = build_schema(
         ctx,
@@ -1707,6 +1812,26 @@ def test_path_parameters_without_constraints_negative(ctx):
     )
 
 
+def test_path_parameters_with_unsupported_regex_pattern(ctx):
+    schema = build_schema(
+        ctx,
+        [
+            {
+                "name": "foo_id",
+                "in": "path",
+                "required": True,
+                "schema": {"pattern": "'^[-._\\p{L}\\p{N}]+$'"},
+            },
+        ],
+        path="/foo/{foo_id}",
+    )
+    assert_negative_coverage(
+        schema,
+        [],
+        ("/foo/{foo_id}", "post"),
+    )
+
+
 def test_query_without_constraints_negative(ctx):
     # When there are no constraints, then we can't generate negative values as everything will match the previous schema, only missing parameter
     schema = build_schema(
@@ -1916,6 +2041,39 @@ def failed(ctx, response, case):
         )
 
 
+@pytest.mark.openapi_version("3.0")
+def test_avoid_testing_unexpected_methods(ctx):
+    raw_schema = {
+        "/foo": {
+            "post": {
+                "parameters": [{"in": "query", "name": "key", "schema": {"type": "integer"}}],
+                "responses": {"200": {"description": "OK"}},
+            },
+            "get": {
+                "responses": {"200": {"description": "OK"}},
+            },
+        }
+    }
+    schema = ctx.openapi.build_schema(raw_schema)
+
+    schema = schemathesis.openapi.from_dict(schema)
+
+    methods = set()
+    operation = schema["/foo"]["post"]
+
+    def test(case):
+        if case.meta.phase.name != TestPhase.COVERAGE:
+            return
+        if not case.meta.phase.data.description.startswith("Unspecified"):
+            return
+        methods.add(case.method)
+        assert f"-X {case.method}" in case.as_curl_command()
+
+    run_negative_test(operation, test, unexpected_methods=set())
+
+    assert not methods
+
+
 def test_missing_authorization(ctx, cli, snapshot_cli, openapi3_base_url):
     # The reproduction code should not contain auth if it is explicitly specified
     schema_path = ctx.openapi.write_schema(
@@ -2007,6 +2165,92 @@ def test_nested_parameters(ctx):
     assert ranges == {"0"}
 
 
+def _request_body(inner):
+    return {
+        "requestBody": {
+            "content": {
+                "application/json": {
+                    "schema": inner,
+                }
+            }
+        }
+    }
+
+
+def _schemas(inner):
+    return {}
+
+
+@pytest.mark.parametrize(
+    ["operation", "components"],
+    [
+        (
+            _request_body(
+                {
+                    "properties": {
+                        "p1": {
+                            "$ref": "#components/schemas/Key",
+                        }
+                    }
+                }
+            ),
+            {
+                "schemas": {
+                    "Key": {
+                        "allOf": [
+                            {"$ref": ""},
+                        ]
+                    }
+                }
+            },
+        ),
+        (
+            _request_body({"$ref": "#components/schemas/Key"}),
+            {
+                "schemas": {
+                    "Key": {
+                        "default": 0,
+                        "items": {
+                            "$ref": "",
+                        },
+                    }
+                }
+            },
+        ),
+        (
+            {"parameters": [{"$ref": "#components/parameters/q"}]},
+            {
+                "parameters": {
+                    "q": {
+                        "in": "header",
+                        "name": "q",
+                        "content": {
+                            "text/plain": {"schema": {"$ref": "#unknown"}},
+                        },
+                    }
+                }
+            },
+        ),
+    ],
+    ids=["body-combinator", "body-items", "parameter-unresolvable"],
+)
+def test_references(ctx, operation, components):
+    raw_schema = ctx.openapi.build_schema({"/test": {"post": operation}}, components=components)
+    schema = schemathesis.openapi.from_dict(raw_schema)
+    for operation in schema.get_all_operations():
+        if isinstance(operation, Ok):
+            for _ in _iter_coverage_cases(
+                operation=operation.ok(),
+                generation_modes=list(GenerationMode),
+                generate_duplicate_query_parameters=False,
+                unexpected_methods=set(),
+                generation_config=schema.config.generation,
+            ):
+                pass
+        else:
+            assert str(operation.err()) == "Schema `` has a required reference to itself"
+
+
 def test_urlencoded_payloads_are_valid(ctx):
     schema = build_schema(
         ctx,
@@ -2038,6 +2282,31 @@ def test_urlencoded_payloads_are_valid(ctx):
     run_test(operation, test)
 
 
+def test_malformed_content_type(ctx):
+    schema = build_schema(
+        ctx,
+        request_body={
+            "required": True,
+            "content": {
+                "invalid": {
+                    "schema": {"type": "object"},
+                }
+            },
+        },
+    )
+    schema = schemathesis.openapi.from_dict(schema)
+
+    operation = schema["/foo"]["post"]
+
+    def test(case):
+        if case.meta.phase != TestPhase.COVERAGE:
+            return
+        assert_requests_call(case)
+
+    with pytest.raises(MalformedMediaType):
+        run_test(operation, test)
+
+
 def test_no_missing_header_duplication(ctx):
     schema = build_schema(
         ctx,
@@ -2194,8 +2463,8 @@ def _validate_serialized_items_are_negative(serialized_items, parameter, case):
         return
 
     # Get the JSON schema for validation
-    schema = parameter.as_json_schema(case.operation)
-    validator = case.operation.schema.validator_cls(
+    schema = parameter.optimized_schema
+    validator = case.operation.schema.adapter.jsonschema_validator_cls(
         schema,
         format_checker=jsonschema.Draft202012Validator.FORMAT_CHECKER,
     )
