@@ -295,9 +295,7 @@ def app_factory(ctx):
             return jsonify({"error": "Something went wrong - PATCH"}), 500
         if user:
             data = request.get_json()
-            if not config.merge_body:
-                assert len(data) == 1
-            else:
+            if config.merge_body:
                 assert "name" in data
                 user["name"] = data["name"]
             return jsonify(user)
