@@ -2,7 +2,6 @@ import platform
 
 import pytest
 
-from schemathesis.core.errors import RECURSIVE_REFERENCE_ERROR_MESSAGE
 from schemathesis.generation.modes import GenerationMode
 
 
@@ -169,7 +168,10 @@ def test_c(case):
     assert settings().deadline.microseconds == 100 * 1000
 
 def test_d():
-    assert settings().deadline.microseconds == 200 * 1000
+    deadline = settings().deadline
+    # E.g. changed globally via a custom profile in Schemathesis' test suite
+    if deadline is not None:
+        assert deadline.microseconds == 200 * 1000
 """,
     )
     # When there is a test with Pytest
@@ -326,8 +328,8 @@ def test(case):
 
 
 @pytest.mark.skipif(platform.system() == "Windows", reason="Fails on Windows due to recursion")
-def test_skip_operations_with_recursive_references(testdir, schema_with_recursive_references):
-    # When the test schema contains recursive references
+def test_fail_on_operations_with_infinite_recursive_references(testdir, schema_with_recursive_references):
+    # When the test schema contains infinite recursive references
     testdir.make_test(
         """
 @schema.parametrize()
@@ -337,9 +339,9 @@ def test(case):
         generation_modes=[GenerationMode.POSITIVE],
     )
     result = testdir.runpytest("-rs")
-    # Then this test should be skipped with a proper error message
-    result.assert_outcomes(skipped=1)
-    assert RECURSIVE_REFERENCE_ERROR_MESSAGE in result.stdout.str()
+    # Then this test should be failed with a proper error message
+    result.assert_outcomes(failed=1)
+    assert "Schema `#/components/schemas/Node` has a required reference to itself" in result.stdout.str()
 
 
 def test_checks_as_a_list(testdir, openapi3_base_url):
@@ -917,7 +919,10 @@ def test_missing_path_parameter(testdir, openapi3_schema_url):
 schema = schemathesis.openapi.from_url('{openapi3_schema_url}')
 
 @schema.parametrize()
+@settings(max_examples=3)
 def test(case):
+    if "id" not in case.path_parameters:
+        pytest.skip("Test should normally fail")
     case.call_and_validate()
 """
     )
