@@ -5,8 +5,7 @@ import pytest
 import schemathesis
 from schemathesis.core.errors import InvalidSchema, LoaderError, OperationNotFound
 from schemathesis.core.result import Err, Ok
-from schemathesis.specs.openapi.parameters import OpenAPI20Body
-from schemathesis.specs.openapi.schemas import InliningResolver
+from schemathesis.specs.openapi.schemas import ReferenceResolver
 
 
 @pytest.mark.parametrize("base_path", ["/v1", "/v1/"])
@@ -46,13 +45,13 @@ def test_open_api_specification(openapi_30):
 
 def test_resolver_cache(simple_schema, mocker):
     schema = schemathesis.openapi.from_dict(simple_schema)
-    spy = mocker.patch("schemathesis.specs.openapi.schemas.InliningResolver", wraps=InliningResolver)
+    spy = mocker.patch("schemathesis.specs.openapi.schemas.ReferenceResolver", wraps=ReferenceResolver)
     assert "_resolver" not in schema.__dict__
-    assert isinstance(schema.resolver, InliningResolver)
+    assert isinstance(schema.resolver, ReferenceResolver)
     assert spy.call_count == 1
     # Cached
     assert "_resolver" in schema.__dict__
-    assert isinstance(schema.resolver, InliningResolver)
+    assert isinstance(schema.resolver, ReferenceResolver)
     assert spy.call_count == 1
 
 
@@ -83,9 +82,11 @@ def test_resolving_multiple_files():
     schema = schemathesis.openapi.from_dict(raw_schema)
     assert len(schema["/teapot"]["post"].body) == 1
     body = schema["/teapot"]["post"].body[0]
-    assert isinstance(body, OpenAPI20Body)
     assert body.media_type == "application/json"
     assert body.definition == {
+        "in": "body",
+        "name": "user",
+        "required": True,
         "schema": {
             "type": "object",
             "properties": {
@@ -100,9 +101,6 @@ def test_resolving_multiple_files():
             },
             "xml": {"name": "User"},
         },
-        "in": "body",
-        "name": "user",
-        "required": True,
     }
 
 
@@ -143,6 +141,25 @@ def test_not_recoverable_schema_error(simple_schema):
         list(schema.get_all_operations())
 
 
+def test_invalid_parameter_schema_type(ctx):
+    # When a parameter's schema is not a dict or bool (e.g., a list)
+    raw_schema = ctx.openapi.build_schema(
+        {
+            "/users": {
+                "get": {
+                    "parameters": [{"in": "query", "name": "filter", "schema": ["invalid", "list"]}],
+                    "responses": {"200": {"description": "OK"}},
+                }
+            }
+        },
+    )
+    schema = schemathesis.openapi.from_dict(raw_schema)
+    operation = schema["/users"]["get"]
+    # Then getting this parameter's schema should raise InvalidSchema
+    with pytest.raises(InvalidSchema, match="Can not generate data for query parameter"):
+        _ = operation.query[0].optimized_schema
+
+
 def test_no_paths_on_openapi_3_1():
     raw_schema = {
         "openapi": "3.1.0",
@@ -234,7 +251,6 @@ def test_get_operation_by_id_in_referenced_path_shared_parameters(ctx):
     operation = schema.get_operation_by_id("getFoo")
     assert operation.path == "/foo"
     assert operation.method.upper() == "GET"
-    assert operation.query.get("foo").definition == parameter
 
 
 def test_get_operation_by_id_no_paths_on_openapi_3_1():
@@ -247,26 +263,6 @@ def test_get_operation_by_id_no_paths_on_openapi_3_1():
         schema.get_operation_by_id("getFoo")
 
 
-@pytest.mark.parametrize(
-    ("fixture", "path"),
-    [
-        ("simple_schema", "/users"),
-        ("simple_openapi", "/query"),
-    ],
-)
-def test_missing_payload_schema(request, fixture, path):
-    raw_schema = request.getfixturevalue(fixture)
-    schema = schemathesis.openapi.from_dict(raw_schema)
-    operation = schema[path]["GET"]
-    assert operation.get_raw_payload_schema("application/xml") is None
-    assert operation.get_resolved_payload_schema("application/xml") is None
-
-
-def test_missing_payload_schema_media_type(open_api_3_schema_with_yaml_payload):
-    schema = schemathesis.openapi.from_dict(open_api_3_schema_with_yaml_payload)
-    assert schema["/yaml"]["POST"].get_raw_payload_schema("application/xml") is None
-
-
 @pytest.mark.skipif(platform.python_implementation() == "PyPy", reason="PyPy behaves differently")
 def test_ssl_error(server):
     with pytest.raises(LoaderError) as exc:
